@@ -15,12 +15,16 @@ package Image::ExifTool::Panasonic;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '1.01';
+$VERSION = '1.03';
 
 %Image::ExifTool::Panasonic::Main = (
+    WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
+    CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
+    WRITABLE => 1,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     0x01 => {
         Name => 'ImageQuality',
+        Writable => 'int16u',
         PrintConv => {
             2 => 'High',
             3 => 'Standard',
@@ -30,11 +34,15 @@ $VERSION = '1.01';
     },
     0x02 => {
         Name => 'FirmwareVersion',
-        Format => 'int8u',
-        PrintConv => '$_=$val; s/ /\./g; $_',
+        Format => 'int8u',  # (format type is 'undef', but it should really be int8u)
+        Writable => 'int8u',
+        Count => 4,
+        PrintConv => '$_=$val; tr/ /./; $_',
+        PrintConvInv => '$_=$val; tr/./ /; $_',
     },
     0x03 => {
         Name => 'WhiteBalance',
+        Writable => 'int16u',
         PrintConv => {
             1 => 'Auto',
             2 => 'Daylight',
@@ -47,6 +55,7 @@ $VERSION = '1.01';
     },
     0x07 => {
         Name => 'FocusMode',
+        Writable => 'int16u',
         PrintConv => {
             1 => 'Auto',
             2 => 'Manual',
@@ -54,6 +63,8 @@ $VERSION = '1.01';
     },
     0x0f => {
         Name => 'SpotMode',
+        Writable => 'int8u',
+        Count => 2,
         PrintConv => {
             '0 1'  => 'Spot Mode On',
             '0 16' => 'Spot Mode Off',
@@ -61,6 +72,7 @@ $VERSION = '1.01';
     },
     0x1a => {
         Name => 'ImageStabilizer',
+        Writable => 'int16u',
         PrintConv => {
             2 => 'On, Mode 1',
             3 => 'Off',
@@ -69,10 +81,12 @@ $VERSION = '1.01';
     },
     0x1c => {
         Name => 'MacroMode',
+        Writable => 'int16u',
         PrintConv => { 1 => 'On', 2 => 'Off' },
     },
     0x1f => {
         Name => 'ShootingMode',
+        Writable => 'int16u',
         PrintConv => {
             1  => 'Normal',
             2  => 'Portrait',
@@ -93,25 +107,32 @@ $VERSION = '1.01';
     },
     0x20 => {
         Name => 'Audio',
+        Writable => 'int16u',
         PrintConv => { 1 => 'Yes', 2 => 'No' },
     },
     0x21 => { #2
         Name => 'DataDump',
+        Writable => 0,
         PrintConv => '\$val',
     },
     0x23 => {
         Name => 'WhiteBalanceBias',
         Format => 'int16s',
+        Writable => 'int16s',
         ValueConv => '$val / 3',
         PrintConv => 'Image::ExifTool::Exif::ConvertFraction($val)',
+        ValueConvInv => '$val * 3',
+        PrintConvInv => 'eval $val',
     },
     0x24 => {
         Name => 'FlashBias',
         Format => 'int16s',
+        Writable => 'int16s',
     },
     0x25 => 'SerialNumber', #2
     0x28 => {
         Name => 'ColorEffect',
+        Writable => 'int16u',
         PrintConv => {
             1 => 'Off',
             2 => 'Warm',
@@ -123,6 +144,8 @@ $VERSION = '1.01';
     # 0x29 => 'SubjectDistance?',
     0x2c => {
         Name => 'Contrast',
+        Flags => 'PrintHex',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'Standard',
             1 => 'Low',
@@ -135,6 +158,7 @@ $VERSION = '1.01';
     },
     0x2d => {
         Name => 'NoiseReduction',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'Standard',
             1 => 'Low',
@@ -144,6 +168,7 @@ $VERSION = '1.01';
     0x0e00 => {
         Name => 'PrintIM',
         Description => 'Print Image Matching',
+        Writable => 0,
         SubDirectory => {
             TagTable => 'Image::ExifTool::PrintIM::Main',
             Start => '$valuePtr',
@@ -171,7 +196,7 @@ Panasonic and Leica maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2004, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2005, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
